#include "../common/common.h"

#define N 100

GLfloat velocity[N][2];
GLubyte color[N][3];

GLuint program;
GLuint vxParam, vyParam, timeParam;


static void init()
{
    glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
    glColor3f(0.0,0.0,0.0);
    glPointSize(10.0);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0.0,1.0,0.0,1.0);

    glEnable(GL_DEPTH_TEST);

    // enable changing the point size in the vertex shader via the gl_PointSize variable;
    glEnable(GL_POINT_SPRITE);
    glEnable(GL_VERTEX_PROGRAM_POINT_SIZE);

    glewInit();
    program = initShader("v.glsl", "f.glsl");

    timeParam = glGetUniformLocation(program, "time");
    vxParam = glGetAttribLocation(program, "vx");
    vyParam = glGetAttribLocation(program, "vy");

    int i, j;
    srand(time(0));

    for(i=0;i<N;i++)
    {
        velocity[i][0] = 0.001 * 0.5 * ((rand()%256)/256.0-0.5) ;
        velocity[i][1] = 0.001 * 1.0 * ((rand()%256)/256.0+0.5) ;
    }

    for(i=0;i<N;i++)
    {
        for(j=0; j<3; j++)
            color[i][j] = rand()%256;
    }

}

static void draw(void)
{
    int i;
    glUniform1f(timeParam, glutGet(GLUT_ELAPSED_TIME));

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glBegin(GL_POINTS);
    for(i=0; i<N; i++)
    {
        glColor3ubv(color[i]);
        glVertexAttrib1f(vxParam, velocity[i][0]);
        glVertexAttrib1f(vyParam, velocity[i][1]);
        glVertex2f(0.5, 0.5);
    }
    glEnd();

    glutSwapBuffers();
    glutPostRedisplay();
}

int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize(512, 512);
    glutCreateWindow("Simple GLSL example");
    glutDisplayFunc(draw);
    glutKeyboardFunc(commonKeyboard);

    init();

    glutMainLoop();
    return 0;
}
